# frozen_string_literal: true

# Licensed to the Software Freedom Conservancy (SFC) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The SFC licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

module Selenium
  module WebDriver
    module WheelActions
      class ScrollOrigin
        class << self
          def element(element, x_offset = 0, y_offset = 0)
            new(element, x_offset, y_offset)
          end

          def viewport(x_offset = 0, y_offset = 0)
            new(:viewport, x_offset, y_offset)
          end
        end

        attr_reader :origin, :x_offset, :y_offset

        #
        # Use a static method to access
        # @api private
        #
        def initialize(origin, x_offset, y_offset)
          @origin = origin
          @x_offset = x_offset
          @y_offset = y_offset
        end
      end # ScrollOrigin
    end # WheelActions
  end # WebDriver
end # Selenium
